import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, max, format, extent, scaleOrdinal } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { ColorLegend } from './ColorLegend';

const width = 960;
const height = 500;
const margin = { top: 20, right: 190, bottom: 100, left: 95 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 60;
const fadeOpacity = 0.2

const App = () => {
  const data = useData();
  const [hoveredValue, setHoveredValue] = useState(null);
 
  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const yValue = d => d.nplay_video;
  const yAxisLabel = 'Video Plays';

  const xValue = d => d.ndays_act;
  const xAxisLabel = 'Days active';

  const colorValue = d => d.Course;
  const colorLegendLabel = 'EdX Course';
  
  const filteredData = data.filter(d => hoveredValue === colorValue(d));
  
  const circleRadius = 7;
  
  const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');
  const yAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0]);
  
  const colorScale = scaleOrdinal ()
  .domain(data.map(colorValue))
  .range(['#c90016 ', '#819c00', '#8E6C8A']);

  console.log(colorScale.domain());
  
  return (
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={5}
        />
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},${innerHeight /
            2}) rotate(-90)`}
        >
          {yAxisLabel}
        </text>
        <AxisLeft 
          yScale={yScale} filtered
          innerWidth={innerWidth} 
          tickOffset={5} />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>
        <g transform={`translate(${innerWidth + 60}, 60)`}>
          <text
            x={35}
            y={-30}
            className="axis-label"
						textAnchor="middle"
        >
          {colorLegendLabel}
        </text>
        <ColorLegend 
          tickSpacing = {22}
          tickSize = {10}
          tickTextOffset = {12}
          tickSize={circleRadius}
          colorScale={colorScale}
          onHover={setHoveredValue}
          hoveredValue={hoveredValue}
          fadeOpacity={fadeOpacity}
            />
          </g>
        <g opacity={hoveredValue ? fadeOpacity : 1}>
        <Marks
          data={data}
          xScale={xScale}
          xValue={xValue}
          yScale={yScale}
    			yValue={yValue}
          colorScale={colorScale}
          colorValue={colorValue}
          tooltipFormat={xAxisTickFormat}
          tooltipFormat={yAxisTickFormat}
          circleRadius={circleRadius}
        />
          </g>
        <Marks
          data={filteredData}
          xScale={xScale}
          xValue={xValue}
          yScale={yScale}
    			yValue={yValue}
          colorScale={colorScale}
          colorValue={colorValue}
          tooltipFormat={xAxisTickFormat}
          tooltipFormat={yAxisTickFormat}
          circleRadius={circleRadius}
        />
      </g>
    </svg>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
